#ifndef _GEOMETRY_H
#define _GEOMETRY_H

// Copyright (c) OpenMedia Lab., 2013. ver.8/14/2013

#include <stdio.h>
#include <math.h>
#include "matrix.h" // vector.h is included in matrix.h

inline double rad( double x ) { return x*1.74532925199433E-2; }
inline double deg( double x ) { return x*5.72957795130824E1; }
int powi( int x, int y ){ int z = 1; for(int i=0; i<y; i++ ) z *= x; return z; }

class Plane{
public:
	Vector normal;
	Vector passage;

	Plane set( double nx, double ny, double nz,  double px, double py, double pz ){ normal.set( nx, ny, nz ); passage.set( px, py, pz ); return *this; }
	Plane set( double a, double b, double c, double d ){
		normal.x = a; normal.y = b; normal.z = c;
		if( fabs( a ) > fabs( b ) && fabs( a ) > fabs( c ) ){
			passage.set( -d / a, 0.0, 0.0 );
		}else if( fabs( b ) > fabs( c ) ){
			passage.set( 0.0, -d / b, 0.0 );
		}else{
			passage.set( 0.0, 0.0, -d / c );
		}
		return *this;
	}
	double signedDistance( Vector &point ){ return ( point - passage ) * normal; }
	double distance( Vector &point ){ return fabs( this->signedDistance( point ) ); }
	void equidistance( Vector &a, Vector &b ){
		normal = b - a;
		passage = a + b;
		passage.multiply( 0.5 );
	}
	void print(){ printf( "normal " ); normal.print(); printf( "passage " ); passage.print(); }
	Plane(){};
	Plane ( double nx, double ny, double nz,  double px, double py, double pz ){ this->set( nx, ny, nz, px, py, pz ); }
	Plane( double a, double b, double c, double d ){ this->set( a, b, c, d ); }
	Plane( Vector &normal, Vector &passage ){ this->normal = normal; this->passage = passage; }
};

void Vector::symmetry( Plane &plane ){
	double t = 2 * ( ( plane.passage - *this ) * plane.normal ) / plane.normal.abs2();
	Vector v = t * plane.normal;
	*this += v;
}

class Line{
public:
	Vector direction;
	Vector passage;

	double distance( Vector &point ){
	Plane plane;
		plane.normal = direction;
		plane.passage = point;
		Vector projected;
		projected.intersection( *this, plane );
		return projected.distance( point );
	}
	void equidistance( Line &a, Line &b ){
		Vector vertical;
			vertical.exterior( a.direction, b.direction );
			Plane pl;
			pl.normal.exterior( a.direction, vertical );
			pl.passage = a.passage;
			a.direction.normalize();
			b.direction.normalize();
			direction = a.direction + b.direction;
			passage.intersection( pl, b );
	}
	void intersection( Plane &a, Plane &b ){
		double ai, bi, d;
		direction.exterior( a.normal, b.normal );
		direction.normalize();
		ai = a.normal * a.passage;
		bi = b.normal * b.passage;
		if( fabs( direction.x ) > fabs( direction.y ) && fabs( direction.x ) > fabs( direction.z ) ){
			passage.x = 0;
			d  = a.normal.y * b.normal.z - a.normal.z * b.normal.y;
			passage.y = ( ai * b.normal.z - bi * a.normal.z) / d;
			passage.z = -( ai * b.normal.y - bi * a.normal.y) / d;
		}else if( fabs( direction.y ) > fabs( direction.z ) ){
			passage.y = 0;
			d  = a.normal.z * b.normal.x - a.normal.x * b.normal.z;
			passage.z = ( ai * b.normal.x - bi * a.normal.x) / d;
			passage.x = -( ai * b.normal.z - bi * a.normal.z) / d;
		}else{
			passage.z = 0;
			d  = a.normal.x * b.normal.y - a.normal.y * b.normal.x;
			passage.x = ( ai * b.normal.y - bi * a.normal.y) / d;
			passage.y = -( ai * b.normal.x - bi * a.normal.x) / d;
		}
	}
	void print(){ printf( "direction " ); direction.print(); printf( "passage " ); passage.print(); }
	void set( Vector &a, Vector &b ){ direction.subtract( b, a ); passage = a; }
	Line(){};
	Line( Vector &a, Vector &b ){ this->set( a, b ); }
};

void Vector::intersection( class Line &line, class Plane &plane ){
	double t = ( plane.passage - line.passage ) * plane.normal;
	t /= plane.normal * line.direction;
	*this = line.direction;
	this->multiply( t );
	this->add( line.passage );
}

void Vector::projection( Line &line ){
	this->subtract( line.passage );
	this->projection( line.direction );
	this->add( line.passage );
}

void Vector::rotate( Line &axis, double angle ){
	this->subtract( axis.passage );
	this->rotate( axis.direction, angle );
	this->add( axis.passage );
}

void Vector::symmetry( Line &line ){
	Vector a;
	a.projection( *this, line );
	a.subtract( *this );
	a.multiply( 2.0 );
	this->add( a );
}

class Line2D{
public:
	Vector2D normal;
	Vector2D direction; // normal and direction are vertical to each other.
	Vector2D passage;

	Line2D set( double nx, double ny, double px, double py ){ normal.set( nx, ny ); direction.set( ny, -nx ); passage.set( px, py ); return *this; }
	Line2D set( double a, double b, double c ){
		normal.x = a; normal.y = b;
		direction.x = normal.y; direction.y = -normal.x;
		if( fabs( a ) > fabs( b ) ){
			passage.set( -c / a, 0.0 );
		}else{
			passage.set( 0.0, -c / b );
		}
		return *this;
	}
	double signedDistance( Vector2D &a ){ return ( a - passage ) * normal; }
//	double distance( Vector2D &point ){ return fabs( this->signedDistance( point ) ); }
	void equidistance( Vector2D &a, Vector2D &b ){
		normal = b - a;
		direction.x = normal.y; direction.y = -normal.x;
		passage = a + b;
		passage.multiply( 0.5 );
	}
	void print(){ printf( "normal " ); normal.print(); printf( "passage " ); passage.print(); }
	Line2D(){};
	Line2D( double nx, double ny, double px, double py ){ this->set( nx, ny, px, py ); }
	Line2D( double a, double b, double c ){ this->set( a, b, c ); }
};

class Sphere{
public:
	Vector center;
	double radius;
};

void Vector::intersectionNear( Line &line, Sphere &sphere ){
Vector vec;
double a, b, c;
	a = line.direction.abs2();
	vec.subtract( line.passage, sphere.center );
	b = line.direction * vec * 2;
	c = vec.abs2() - sphere.radius * sphere.radius;
	Vector2D p;
	p.squareEquation( a, b, c );
	this->multiply( line.direction, p.x );
	this->add( line.passage );
}

void Vector::intersectionFar( Line &line, Sphere &sphere ){
Vector vec;
double a, b, c;
	a = line.direction.abs2();
	vec.subtract( line.passage, sphere.center );
	b = line.direction * vec * 2;
	c = vec.abs2() - sphere.radius * sphere.radius;
	Vector2D p;
	p.squareEquation( a, b, c );
	this->multiply( line.direction, p.y );
	this->add( line.passage );
}

#endif //_GEOMETRY_H
