#ifndef _VECTOR_H
#define _VECTOR_H

// Copyright (c) OpenMedia Lab., 2013. ver.8/14/2013

#include <stdio.h>
#include <math.h>
#include <float.h> // to use DBL_EPSILON, non zero minimum of double

class Vector{
public:
	double x;
	double y;
	double z;

	Vector( double a, double b, double c ){ x=a; y=b; z=c; }
	Vector( double a, double b ){ x=a; y=b; z=0; }
	Vector(){}
	double abs(){ return sqrt(x*x + y*y + z*z); }
	double abs2(){ return x*x + y*y + z*z; }
	void   add( Vector &a ){ x+=a.x; y+=a.y; z+=a.z; }
	void   add( Vector &a, Vector &b ){ x=a.x+b.x; y=a.y+b.y; z=a.z+b.z; }
	void   add( double a, double b, double c ){ x+=a; y+=b; z+=c; }
	void   affine( class Matrix4X4 &matrix, Vector &a );
	void   affine( Vector &a, class Matrix4X4 &matrix );
	void   affine( class Matrix4X4 &matrix ){ Vector a = *this; this->affine( matrix, a ); }
	double angle( Vector &a ){ return acos(a * *this / sqrt(a.abs2() * this->abs2()) ); }
	void   set( double a, double b, double c ){ x=a; y=b; z=c; }
	void   set( double a, double b ){ x=a; y=b; }
	void   set( class Vector2D &a );
	void   setPolar( double lng, double lat ){ x=cos(lng)*cos(lat); y=sin(lng)*cos(lat); z=sin(lat); }
	double distance( Vector &a ){ return sqrt((x-a.x)*(x-a.x) + (y-a.y)*(y-a.y) + (z-a.z)*(z-a.z)); }
	double distance2( Vector &a ){ return (x-a.x)*(x-a.x) + (y-a.y)*(y-a.y) + (z-a.z)*(z-a.z); }
	int    divide(  Vector &a, Vector &b, double m, double n ){
		if( fabs(m+n) < DBL_EPSILON ){
			printf("Unexpected ratio %f : %f in Vector::divide()!\n", m, n );
			return 1;
		}
		Vector c;
		c.multiply( n, a );
		this->multiply( m, b );
		this->add( c );
		this->multiply( 1.0/(m+n) );
		return 0;
	}
	int    divide( double m, double n, Vector &a, Vector &b ){ return this->divide( a, b, m, n ); }
	void   exchange( Vector &a ){ Vector b = a; a = *this; *this = b; }
	void   exterior( Vector &a, Vector &b ){ x=a.y*b.z-b.y*a.z; y=a.z*b.x-b.z*a.x; z=a.x*b.y-b.x*a.y; }
	int    ifZero(){ return this->abs2() < DBL_EPSILON ? 1 : 0; }
	void   intersectionNear( class Line &line, class Sphere &sphere );
	void   intersectionNear( class Sphere &sphere, class Line &line ){ this->intersectionNear( line, sphere ); }
	void   intersectionFar( class Line &line, class Sphere &sphere );
	void   intersectionFar( class Sphere &sphere, class Line &line ){ this->intersectionFar( line, sphere ); }
	void   intersection( class Line &line, class Plane &plane );
	void   intersection( class Plane &plane, class Line &line ){ this->intersection( line, plane ); }
	double latitude(){ return asin( z/sqrt(x*x+y*y) ); }
	double longitude(){ return atan2( y, x ); }
	void   linear( class Matrix4X4 &matrix, Vector &a );
	void   linear( Vector &a, class Matrix4X4 &matrix );
	void   linear( class Matrix4X4 &matrix ){ Vector a = *this; this->linear( matrix, a ); }
	void   linear( class Matrix3X3 &matrix, Vector &a );
	void   linear( Vector &a, class Matrix3X3 &matrix );
	void   linear( class Matrix3X3 &matrix ){ Vector a = *this; this->linear( matrix, a ); }
	void   multiply( double k ){ x*=k; y*=k; z*=k; }
	void   multiply( double k, Vector &a ){ x=k*a.x; y=k*a.y; z=k*a.z; }
	void   multiply( Vector &a, double k ){ this->multiply( k, a ); }
	void   normal( Vector &a, Vector &b, Vector &c ){ Vector d1, d2; d1.subtract( b, a ); d2.subtract( c, b ); this->exterior( d1, d2 ); }
	void   normalize(){ this->multiply( 1.0 / this->abs() ); }
	void   normalize( Vector &a ){ this->multiply( 1.0 / a.abs(), a ); }
	void   print(){ printf( "%f %f %f\n", x, y, z ); }
	void   projection( Vector &a, Vector &b ){ this->multiply( a*b/b.abs2(), b ); }
	void   projection( Vector &a ){ this->projection( *this, a ); }
	void   projection( class Line &line );
	void   projection( Vector &a, class Line &line ){ *this = a; this->projection( line ); }
	void   projection( class Line &line, Vector &a ){ this->projection( a, line ); }
	void   reverse(){ x = -x; y = -y; z = -z; }
	void   reverse( Vector &a ){ x = -a.x; y = -a.y; z = -a.z; }
	void   rotate( Vector &axis, double angle ){
		Vector projected, uaxis, vaxis;
		projected.projection( *this, axis );
		uaxis.subtract( *this, projected );
		vaxis.exterior( axis, uaxis );
		uaxis.multiply( cos( angle ) );
		vaxis.multiply( sin( angle )/axis.abs() );
		this->add( uaxis, vaxis );
		this->add( projected );
	}
	void   rotate( double angle, Vector &axis ){ this->rotate( axis, angle ); }
	void   rotate( Vector &a, Vector &axis, double angle ){ *this = a; this->rotate( axis, angle ); }
	void   rotate( class Line &axis, double angle );
	void   rotate( Vector &a, class Line &axis, double angle ){ *this = a; this->rotate( axis, angle ); }
	void   rotateX( Vector &a, double ang ){ double c=cos(ang); double s=sin(ang); x=a.x; y=c*a.y-s*a.z; z=s*a.y+c*a.z; }
	void   rotateX( double ang, Vector &a ){ this->rotateX( a, ang ); }
	void   rotateX( double ang ){ Vector a; a= *this; this->rotateX( a, ang ); }
	void   rotateY( Vector &a, double ang ){ double c=cos(ang); double s=sin(ang); y=a.y; z=c*a.z-s*a.x; x=s*a.z+c*a.x; }
	void   rotateY( double ang, Vector &a ){ this->rotateY( a, ang ); }
	void   rotateY( double ang ){ Vector a; a= *this; this->rotateY( a, ang ); }
	void   rotateZ( Vector &a, double ang ){ double c=cos(ang); double s=sin(ang); z=a.z; x=c*a.x-s*a.y; y=s*a.x+c*a.y; }
	void   rotateZ( double ang, Vector &a ){ this->rotateZ( a, ang ); }
	void   rotateZ( double ang ){ Vector a; a= *this; this->rotateZ( a, ang ); }
	void   scale( double a, double b, double c ){ x*=a; y*=b; z*=c; }
	void   subtract( Vector &a ){ x-=a.x; y-=a.y; z-=a.z; }
	void   subtract( Vector &a, Vector &b ){ x=a.x-b.x; y=a.y-b.y; z=a.z-b.z; }
	void   subtract( double a, double b, double c ){ x-=a; y-=b; z-=c; }
	void   symmetry( Vector &a ){ this->reverse(); this->add( a ); this->add( a ); }
	void   symmetry( class Plane &plane );
	void   symmetry( class Plane &plane, Vector &a ){ *this = a; this->symmetry( plane ); }
	void   symmetry( Vector &a, class Plane &plane ){ this->symmetry( plane, a ); }
	void   symmetry( class Line &line );
	void   symmetry( class Line &line, Vector &a ){ *this = a; this->symmetry( line ); }
	void   symmetry( Vector &a, class Line &line ){ this->symmetry( line, a ); }
	void   unitX(){ this->set( 1.0, 0.0, 0.0 ); }
	void   unitY(){ this->set( 0.0, 1.0, 0.0 ); }
	void   unitZ(){ this->set( 0.0, 0.0, 1.0 ); }
	void   zero(){ this->set( 0.0, 0.0, 0.0 ); }
	void   _fwrite( FILE *fp ){
		fwrite( &x, 8, 1, fp );
		fwrite( &y, 8, 1, fp );
		fwrite( &z, 8, 1, fp );
	}
	void   _fread( FILE *fp ){
		fread( &x, 8, 1, fp );
		fread( &y, 8, 1, fp );
		fread( &z, 8, 1, fp );
	}

#ifdef __GL_H__
	void   glNormal(){ glNormal3dv( (GLdouble *)this ); }
	void   glVertex(){ glVertex3dv( (GLdouble *)this ); }
	void   glTranslate(){ glTranslated( GLdouble(x), GLdouble(y), GLdouble(z) ); }
	void   _glutSolidSphere(GLdouble radius, GLint slices, GLint stacks){
		glPushMatrix();
		this->glTranslate();
		glutSolidSphere(radius, slices, stacks);
		glPopMatrix();
	}
#endif
	Vector operator +( Vector &a ){ Vector b; b.add( *this, a ); return b; }
	Vector operator +=( Vector &a ){ this->add( a ); return *this; }
	Vector operator -( Vector &a ){ Vector b; b.subtract( *this, a ); return b; }
	Vector operator -=( Vector &a ){ this->subtract( a ); return *this; }
	Vector operator -(){ Vector a; a.reverse( *this ); return a; }
	double  operator *( Vector &a ){ return x*a.x + y*a.y + z*a.z; }
	Vector operator *( double k ){ Vector a; a.multiply( k, *this ); return a; }
	Vector operator /( double k ){ Vector a; a.multiply( 1.0/k, *this ); return a; }
	friend Vector operator *( double k, Vector &a ){ return a * k; }
	Vector operator *( class Matrix4X4 &m ){ Vector a; a.affine( *this, m ); return a; }
	Vector operator *( class Matrix3X3 &m ){ Vector a; a.linear( m, *this ); return a; }
	Vector operator *=( double k ){ this->multiply( k ); return *this; }
	Vector operator /=( double k ){ this->multiply( 1.0/k ); return *this; }
	Vector operator *=( class Matrix4X4 &m ){ this->affine( m ); return *this; }
	Vector operator &( Vector &a ){ Vector b; b.exterior( *this, a ); return b; }
	Vector operator =( class Vector2D &a ){ this->set( a ); return *this; }
};

class Vector2D{
public:
	double x;
	double y;

	Vector2D( double a, double b ){ x=a; y=b; }
	Vector2D(){}
	double abs(){ return sqrt(x*x + y*y); }
	double abs2(){ return x*x + y*y; }
	void   add( Vector2D &a ){ x+=a.x; y+=a.y; }
	void   add( Vector2D &a, Vector2D &b ){ x=a.x+b.x; y=a.y+b.y; }
	void   add( double a, double b ){ x+=a; y+=b; }
	void   affine( class Matrix3X3 &m, Vector2D &a );
	void   affine( Vector2D &a, class Matrix3X3 &m );
	void   affine( class Matrix3X3 &matrix ){ Vector2D a = *this; this->affine( matrix, a ); }
	double angle( Vector2D &a ){ return acos(a * *this / sqrt(a.abs2() * this->abs2()) ); }
	void   set( double a, double b ){ x=a; y=b; }
	void   setPolar( double lng ){ x=cos(lng); y=sin(lng); }
	double distance( Vector2D &a ){ return sqrt((x-a.x)*(x-a.x) + (y-a.y)*(y-a.y)); }
	double distance2( Vector2D &a ){ return (x-a.x)*(x-a.x) + (y-a.y)*(y-a.y); }
	int    divide(  Vector2D &a, Vector2D &b, double m, double n ){
		if( fabs(m+n) < DBL_EPSILON ){
			printf("Unexpected ratio %f : %f in Vector2D::divide()!\n", m, n );
			return 1;
		}
		Vector2D c;
		c.multiply( n, a );
		this->multiply( m, b );
		this->add( c );
		this->multiply( 1.0/(m+n) );
		return 0;
	}
	int    divide( double m, double n, Vector2D &a, Vector2D &b ){ return this->divide( a, b, m, n ); }
	void   exchange( Vector2D &a ){ Vector2D b = a; a = *this; *this = b; }
	int    ifZero(){ return this->abs2() < DBL_EPSILON ? 1 : 0; }
	void   linear( class Matrix3X3 &matrix, Vector2D &a );
	void   linear( Vector2D &a, class Matrix3X3 &matrix );
	void   linear( class Matrix3X3 &matrix ){ Vector2D a = *this; this->linear( matrix, a ); }
	void   linear( class Matrix2X2 &matrix, Vector2D &a );
	void   linear( Vector2D &a, class Matrix2X2 &matrix );
	void   linear( class Matrix2X2 &matrix ){ Vector2D a = *this; this->linear( matrix, a ); }
	double longitude(){ return atan2( y, x ); }
	void   multiply( double k ){ x*=k; y*=k; }
	void   multiply( double k, Vector2D &a ){ x=k*a.x; y=k*a.y; }
	void   multiply( Vector2D &a, double k ){ this->multiply( k, a ); }
	void   normalize(){ this->multiply( 1.0 / this->abs() ); }
	void   normalize( Vector2D &a ){ this->multiply( 1.0 / a.abs(), a ); }
	void   print(){ printf( "%f %f\n", x, y ); }
	void   projection( Vector2D &a, Vector2D &b ){ this->multiply( a*b/b.abs2(), b ); }
	void   reverse(){ x = -x; y = -y; }
	void   reverse( Vector2D &a ){ x = -a.x; y = -a.y; }
	void   rotate( Vector2D &a, double ang ){ double c=cos(ang); double s=sin(ang); x=c*a.x-s*a.y; y=s*a.x+c*a.y; }
	void   rotate( double ang, Vector2D &a ){ this->rotate( a, ang ); }
	void   rotate( double ang ){ Vector2D a; a= *this; this->rotate( a, ang ); }
	void   scale( double a, double b ){ x*=a; y*=b; }
	void   subtract( Vector2D &a ){ x-=a.x; y-=a.y; }
	void   subtract( Vector2D &a, Vector2D &b ){ x=a.x-b.x; y=a.y-b.y; }
	void   subtract( double a, double b ){ x-=a; y-=b; }
	void   unitX(){ this->set( 1.0, 0.0 ); }
	void   unitY(){ this->set( 0.0, 1.0 ); }
	void   zero(){ this->set( 0.0, 0.0 ); }
	double squareEquation( double a, double b, double c ){
		double d, d2;
		if(fabs(a)<DBL_EPSILON){
			if(fabs(b)<DBL_EPSILON){
				printf("a=b=0 in square Equation!\n"); return 0.0;
			}else{
				x = y = -c/b; return 0.0;
			}
		}
		if((d2 = b * b - 4.0 * a * c) < 0.0) return d2;
		d = sqrt(d2);
		x = (-b - d)/(2.0 * a);
		y  = (-b + d)/(2.0 * a);
		return d2;
	}
#ifdef __GL_H__
	void   glVertex(){ glVertex2dv( (GLdouble *)this ); }
	void   glTexCoord(){ glTexCoord2dv( (GLdouble *)this ); }
#endif
	Vector2D operator +( Vector2D &a ){ Vector2D b; b.add( *this, a ); return b; }
	Vector2D operator +=( Vector2D &a ){ this->add( a ); return *this; }
	Vector2D operator -( Vector2D &a ){ Vector2D b; b.subtract( *this, a ); return b; }
	Vector2D operator -=( Vector2D &a ){ this->subtract( a ); return *this; }
	Vector2D operator -(){ Vector2D a; a.reverse( *this ); return a; }
	double   operator *( Vector2D &a ){ return x*a.x + y*a.y; }
	Vector2D operator *( double k ){ Vector2D a; a.multiply( k, *this ); return a; }
	Vector2D operator /( double k ){ Vector2D a; a.multiply( 1.0/k, *this ); return a; }
	friend Vector2D operator *( double k, Vector2D &a ){ return a * k; }
	Vector2D operator *( class Matrix3X3 &m ){ Vector2D a; a.affine( *this, m ); return a; }
	friend Vector2D operator *( class Matrix3X3 &m, Vector2D &a ){ Vector2D b; b.affine( m, a ); return b; }
	Vector2D operator *=( double k ){ this->multiply( k ); return *this; }
	Vector2D operator /=( double k ){ this->multiply( 1.0/k ); return *this; }
	Vector2D operator *=( class Matrix3X3 &m ){ this->affine( m ); return *this; }
	Vector2D operator =( class Vector &a ){ x = a.x; y = a.y; return *this; }
};

void Vector::set( class Vector2D &a ){ x = a.x; y = a.y; }

#endif //_VECTOR_H
